; Test Position by Width
; By Omegamatrix

    processor 6502
    LIST OFF
    include vcs.h
    LIST ON

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      CONSTANTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ONE_COPY             = 0
TWO_COPIES_CLOSE     = 1
TWO_COPIES_MED       = 2
THREE_COPIES_CLOSE   = 3
TWO_COPIES_WIDE      = 4
DOUBLE_SIZE          = 5
THREE_COPIES_MED     = 6
QUAD_SIZE            = 7

TIME_VBLANK          = 45
TIME_OVERSCAN        = 24

COL_BACKGROUND       = $0E

TOP_LINES            = 88
BOTTOM_LINES         = 90

SPEED_MASK           = $0F


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      RIOT RAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG.U RIOT_RAM
       ORG $80

frameCounter         ds 1
sizeVar              ds 1
delayPos             ds 2


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      MAIN PROGRAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    seg CODE
    ORG $F000

START:
    cld
.loopClear:
    ldx    #$0A          ; ASL opcode = $0A
    inx
    txs
    pha
    bne    .loopClear+1  ; jump between operator and operand to do ASL

    lda    #COL_BACKGROUND
    sta    COLUBK
    
    lda    #>PosPx_21
    sta    delayPos+1
    lda    #<PosPx_21
    sta    delayPos
    
    

MainLoop:
    lda    #$0E
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync

    lda    #TIME_VBLANK
    sta    TIM64T

    
    ldx    #0
    jsr    CoarsePos
    inx
    lda    sizeVar
    sta    NUSIZ0
    jsr    CoarsePos

.waitVblankLogo:
    lda    INTIM
    bne    .waitVblankLogo

    sta    WSYNC
;---------------------------------------
    sta    VBLANK


    ldy    #TOP_LINES
.loopDelayTop:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopDelayTop

    dey
    sty    GRP1        ; Y=$FF
    
    ldx    #10
.loopP1:
    sta    WSYNC
    dex
    bne    .loopP1
    
    stx    GRP1        ; Y=0
    
    sta    WSYNC
    sta    WSYNC
    
    sty    GRP0
    
    ldx    #10
.loopP0:
    sta    WSYNC
    dex
    bne    .loopP0
    
    stx    GRP0        ; Y=0
    


    ldy    #BOTTOM_LINES
.loopDelayBottom:
    sta    WSYNC
;---------------------------------------
    lda    #0
    dey
    bne    .loopDelayBottom


    lda    #2
    sta    WSYNC
;----------------------------------------
    sta    VBLANK
    lda    #TIME_OVERSCAN
    sta    TIM64T
    
    inc    frameCounter
    
    lda    frameCounter
    and    #SPEED_MASK
    bne    .skipDirections
    
    ldx    delayPos
    ldy    sizeVar
    lda    SWCHA
    asl    ;right
    bcs    .checkLeft
    dex
    cpx    #<PosPx_76-1
    bne    .checkLeft
    ldx    #<PosPx_21
.checkLeft:
    asl    ;left
    bcs    .checkDown
    inx
    cpx    #<PosPx_21+1
    bne    .checkDown
    ldx    #<PosPx_76
.checkDown:
    asl    ;down
    bcs    .checkUp
    dey
.checkUp:
    asl    ;up
    bcs    .storeVars
    iny
.storeVars:
    tya
    and    #$07
    sta    sizeVar
    stx    delayPos 
.skipDirections:

.waitOverscanLogo:
    lda    INTIM
    bne    .waitOverscanLogo
    jmp    MainLoop
    
    

    align 256
    
CoarsePos; SUBROUTINE
    sta    WSYNC
;--------------------------------
    dec    $2E
    nop
    nop
    nop
    jmp.ind (delayPos)

PosPx_76:
   .byte $80
PosPx_75:
   .byte $80
PosPx_74:
   .byte $80
PosPx_73:
   .byte $80
PosPx_72:
   .byte $80
PosPx_71:
   .byte $80
PosPx_70:
   .byte $80
PosPx_69:
   .byte $80
PosPx_68:
   .byte $80
PosPx_67:
   .byte $80
PosPx_66:
   .byte $80
PosPx_65:
   .byte $80
PosPx_64:
   .byte $80
PosPx_63:
   .byte $80
PosPx_62:
   .byte $80
PosPx_61:
   .byte $80
PosPx_60:
   .byte $80
PosPx_59:
   .byte $80
PosPx_58:
   .byte $80
PosPx_57:
   .byte $80
PosPx_56:
   .byte $80
PosPx_55:
   .byte $80
PosPx_54:
   .byte $80
PosPx_53:
   .byte $80
PosPx_52:
   .byte $80
PosPx_51:
   .byte $80
PosPx_50:
   .byte $80
PosPx_49:
   .byte $80
PosPx_48:
   .byte $80
PosPx_47:
   .byte $80
PosPx_46:
   .byte $80
PosPx_45:
   .byte $80
PosPx_44:
   .byte $80
PosPx_43:
   .byte $80
PosPx_42:
   .byte $80
PosPx_41:
   .byte $80
PosPx_40:
   .byte $80
PosPx_39:
   .byte $80
PosPx_38:
   .byte $80
PosPx_37:
   .byte $80
PosPx_36:
   .byte $80
PosPx_35:
   .byte $80
PosPx_34:
   .byte $80
PosPx_33:
   .byte $80
PosPx_32:
   .byte $80
PosPx_31:
   .byte $80
PosPx_30:
   .byte $80
PosPx_29:
   .byte $80
PosPx_28:
   .byte $80
PosPx_27:
   .byte $80
PosPx_26:
   .byte $80
PosPx_25:
   .byte $80
PosPx_24:
   .byte $80
PosPx_23:
   .byte $80
PosPx_22:
    .byte $04
PosPx_21:
    nop
    sta    RESP0,X
    rts

      ORG $FFFC

    .word START
    .word START